// -*- C++ -*-
#include "Rivet/Analysis.hh"
#include "Rivet/Projections/FinalState.hh"
#include "Rivet/Projections/UnstableParticles.hh"

namespace Rivet {


  /// @brief Cross section for D0 D*- pi+ +c.c
  class BESIII_2018_I1685535 : public Analysis {
  public:

    /// Constructor
    RIVET_DEFAULT_ANALYSIS_CTOR(BESIII_2018_I1685535);


    /// @name Analysis methods
    /// @{

    /// Book histograms and initialise projections before the run
    void init() {

      // Initialise and register projections
      declare(FinalState(), "FS");
      declare(UnstableParticles(), "UFS");
      // Book histograms
      for (size_t ix=0; ix<2; ++ix) {
        book(_nD0[ix], 1+ix, 1, 1);
        for (const string& en : _nD0[ix].binning().edges<0>()) {
          const double end = stod(en)*GeV;
          if (isCompatibleWithSqrtS(end)) {
            _sqs[ix] = en; break;
          }
        }
      }
      raiseBeamErrorIf(_sqs[0].empty() && _sqs[1].empty());
    }

    void findChildren(const Particle& p,map<long,int>& nRes, int& ncount) const {
      for (const Particle &child : p.children()) {
        if (child.children().empty()) {
          nRes[child.pid()]-=1;
          --ncount;
        }
        else {
          findChildren(child,nRes,ncount);
        }
      }
    }

    /// Perform the per-event analysis
    void analyze(const Event& event) {
      const FinalState& fs = apply<FinalState>(event, "FS");

      map<long,int> nCount;
      int ntotal(0);
      for (const Particle& p : fs.particles()) {
        nCount[p.pid()] += 1;
        ++ntotal;
      }
      const FinalState& ufs = apply<FinalState>(event, "UFS");


      for (size_t ix=0; ix<ufs.particles().size(); ++ix) {
        const Particle& p1 = ufs.particles()[ix];
        if (p1.abspid() != 421) continue;
        map<long,int> nRes = nCount;
        int ncount = ntotal;
        findChildren(p1,nRes,ncount);
        bool matched=false;
        int id2 = p1.pid()>0 ? -413 :  413;
        int ipi = p1.pid()>0 ?  211 : -211;
        for (size_t iy=0; iy<ufs.particles().size(); ++iy) {
          if (ix==iy) continue;
          const Particle& p2 = ufs.particles()[iy];
          if (p2.pid()!=id2) continue;
          map<long,int> nRes2 = nRes;
          int ncount2 = ncount;
          findChildren(p2,nRes2,ncount2);
          if (ncount2!=1) continue;
          matched=true;
          for (const auto& val : nRes2) {
            if (val.first==ipi) {
              if (val.second!=1) {
                matched = false;
                break;
              }
            }
            else if (val.second!=0) {
              matched = false;
              break;
            }
          }
          if (matched) break;
        }
        if (matched) {
          for (size_t ix=0; ix<2; ++ix) {
            if(!_sqs[ix].empty()) _nD0[ix]->fill(_sqs[ix]);
          }
        }
      }
    }


    /// Normalise histograms etc., after the run
    void finalize() {
      scale(_nD0, crossSection()/ sumOfWeights() /nanobarn);
    }

    /// @}


    /// @name Histograms
    /// @{
    BinnedHistoPtr<string> _nD0[2];
    string _sqs[2];
    /// @}


  };


  RIVET_DECLARE_PLUGIN(BESIII_2018_I1685535);

}
